@@ -1,23 +1,18 @@
 # This script generates the PyQt configuration and generates the Makefiles.
 #
-# Copyright (c) 2013 Riverbank Computing Limited <info@riverbankcomputing.com>
+# Copyright (c) 2015 Riverbank Computing Limited <info@riverbankcomputing.com>
 # 
-# This file is part of PyQt.
+# This file is part of PyQt4.
 # 
-# This file may be used under the terms of the GNU General Public
-# License versions 2.0 or 3.0 as published by the Free Software
-# Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-# included in the packaging of this file.  Alternatively you may (at
-# your option) use any later version of the GNU General Public
-# License if such license has been publicly approved by Riverbank
-# Computing Limited (or its successors, if any) and the KDE Free Qt
-# Foundation. In addition, as a special exception, Riverbank gives you
-# certain additional rights. These rights are described in the Riverbank
-# GPL Exception version 1.1, which can be found in the file
-# GPL_EXCEPTION.txt in this package.
+# This file may be used under the terms of the GNU General Public License
+# version 3.0 as published by the Free Software Foundation and appearing in
+# the file LICENSE included in the packaging of this file.  Please review the
+# following information to ensure the GNU General Public License version 3.0
+# requirements will be met: http://www.gnu.org/copyleft/gpl.html.
 # 
-# If you are unsure which license is appropriate for your use, please
-# contact the sales department at sales@riverbankcomputing.com.
+# If you do not wish to use this file under the terms of the GPL version 3.0
+# then you may purchase a commercial license.  For more information contact
+# info@riverbankcomputing.com.
 # 
 # This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 # WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -33,10 +28,10 @@ import sipconfig
 
 
 # Initialise the globals.
-pyqt_version = 0x040a03
-pyqt_version_str = "4.10.3"
+pyqt_version = 0x040b04
+pyqt_version_str = "4.11.4"
 
-sip_min_version = 0x040f00
+sip_min_version = 0x041004
 
 qt_version = 0
 qt_edition = ""
@@ -49,7 +44,7 @@ qt_datadir = None
 qt_archdatadir = None
 qt_pluginsdir = None
 qt_xfeatures = None
-qt_shared = ""
+qt_shared = ''
 qt_framework = 0
 
 # This default value will be used by the code that bootstraps the Qt
@@ -210,10 +205,10 @@ def create_optparser():
 
     if sys.platform == 'darwin':
         g = optparse.OptionGroup(p, title="MacOS X Configuration")
-        g.add_option("--use-arch", action="store", metavar="ARCH",
+        g.add_option("--use-arch", action="append", metavar="ARCH",
                 dest="use_arch", choices=["i386", "x86_64", "ppc"],
-                help="the architecture to use when running pyuic4 "
-                        "[default: system default]")
+                help="add ARCH to the list of architectures to use when "
+                        "running pyuic4 [default: system default]")
         p.add_option_group(g)
 
     # Installation.
@@ -323,7 +318,8 @@ class ConfigurePyQt4:
             0x040800: "Qt_4_7_2",
             0x040803: "Qt_4_8_0",
             0x040804: "Qt_4_8_3",
-            0x050000: "Qt_4_8_4",
+            0x040806: "Qt_4_8_4",
+            0x050000: "Qt_4_8_6",
             0x060000: "Qt_5_0_0"
         }
 
@@ -377,7 +373,7 @@ class ConfigurePyQt4:
                 "new QAssistantClient(\"foo\")", extra_lib_dirs=ass_lib_dirs,
                 extra_libs=ass_libs)
 
-        if not qt_shared:
+        if qt_shared == '':
             sipconfig.inform("QtDesigner module disabled with static Qt libraries.")
         else:
             check_module("QtDesigner", "QExtensionFactory",
@@ -471,8 +467,6 @@ class ConfigurePyQt4:
                         extra_lib_dirs=[qpy_lib_dir], extra_libs=[qpy_lib])
 
         if "QtOpenGL" in pyqt_modules:
-            generate_OpenGL_extras()
-
             qpy_inc_dir, qpy_lib_dir, qpy_lib = self._qpy_directories("QtOpenGL", "qpyopengl")
 
             if opts.bigqt:
@@ -883,7 +877,11 @@ include(%s)
 
         if sys.platform == 'darwin':
             gui = True
-            use_arch = opts.use_arch
+
+            if opts.use_arch is None:
+                use_arch = ''
+            else:
+                use_arch = ' '.join(opts.use_arch)
         else:
             gui = False
             use_arch = ''
@@ -977,6 +975,8 @@ include(%s)
                 if dynamic_pylib:
                     if glob.glob("%s/lib/libpython%d.%d*" % (ducfg["exec_prefix"], py_major, py_minor)):
                         lib_dir_flag = quote("-L%s/lib" % ducfg["exec_prefix"])
+                    elif 'MULTIARCH' in ducfg and glob.glob('%s/lib/%s/libpython%d.%d*' % (ducfg['exec_prefix'], ducfg['MULTIARCH'], py_major, py_minor)):
+                        lib_dir_flag = quote('-L%s/lib/%s' % (ducfg['exec_prefix'], ducfg['MULTIARCH']))
                     elif glob.glob("%s/libpython%d.%d*" % (ducfg["LIBDIR"], py_major, py_minor)):
                         lib_dir_flag = quote("-L%s" % ducfg["LIBDIR"])
                     else:
@@ -986,7 +986,7 @@ include(%s)
 
                     link = "%s -lpython%d.%d%s" % (lib_dir_flag, py_major, py_minor, abi)
                 else:
-                    sipconfig.inform("Qt Designer plugin disabled because Python library is static")
+                    sipconfig.inform("The Qt Designer plugin was disabled because a dynamic Python library couldn't be found")
                     opts.designer_plugin = False
 
                 pysh_lib = ducfg.get("LDLIBRARY", "")
@@ -1002,7 +1002,7 @@ include(%s)
                 os.chdir("designer")
 
                 # Create the qmake project file.
-                fin = open(os.path.join(src_dir, "designer", "python.pro-in"))
+                fin = open(os.path.join(src_dir, "designer", "designer.pro-in"))
                 prj = fin.read()
                 fin.close()
 
@@ -1012,7 +1012,7 @@ include(%s)
                 prj = prj.replace("@PYSHLIB@", pysh_lib)
                 prj = prj.replace("@QTPLUGINDIR@", quote(opts.plugindir + "/designer"))
 
-                fout = open("python.pro", "w+")
+                fout = open("designer.pro", "w+")
 
                 if sipcfg.arch:
                     fout.write(arch_config())
@@ -1194,73 +1194,6 @@ def remove_file(fname):
         pass
 
 
-def generate_OpenGL_extras():
-    """Generate the extras needed by the QtOpenGL module (i.e. the .sip file
-    defining the correct typedefs for the OpenGL data types.
-    """
-    sipconfig.inform("Determining the OpenGL data types...")
-
-    src = "opengl_extras.cpp"
-
-    f = open(src, "w")
-
-    f.write(
-"""#include <QFile>
-#include <QTextStream>
-#include <qgl.h>
-
-int main(int, char **)
-{
-    QFile outf("./sip/QtOpenGL/opengl_types.sip");
-
-    if (!outf.open(QIODevice::WriteOnly|QIODevice::Truncate|QIODevice::Text))
-        return 1;
-
-    QTextStream out(&outf);
-
-    if (sizeof (long) == sizeof (GLint))
-        out << "typedef long GLint;\\n";
-    else
-        out << "typedef int GLint;\\n";
-
-    if (sizeof (unsigned long) == sizeof (GLuint))
-        out << "typedef unsigned long GLuint;\\n";
-    else
-        out << "typedef unsigned GLuint;\\n";
-
-    if (sizeof (unsigned long) == sizeof (GLenum))
-        out << "typedef unsigned long GLenum;\\n";
-    else
-        out << "typedef unsigned GLenum;\\n";
-
-    if (sizeof (unsigned long) == sizeof (GLbitfield))
-        out << "typedef unsigned long GLbitfield;\\n";
-    else
-        out << "typedef unsigned GLbitfield;\\n";
-
-    out << "typedef float GLfloat;\\n";
-
-    return 0;
-}
-""")
-
-    f.close()
-
-    cmd = compile_qt_program(src, "QtOpenGL")
-
-    if cmd is None:
-        sipconfig.error("Unable to determine the OpenGL data types.")
-
-    # SIP's build system (specifically ProgramMakefile.build_command()) assumes
-    # that an executable can be created by running a single command.  In the
-    # case of MSVC this is incorrect.  As a quick hack we do the extra step
-    # here if it looks like it is needed.
-    if os.access("opengl_extras.manifest", os.F_OK):
-        run_command("mt -nologo -manifest opengl_extras.manifest -outputresource:opengl_extras.exe;1")
-
-    run_command(cmd)
-
-
 def compile_qt_program(name, mname, extra_include_dirs=None, extra_lib_dirs=None, extra_libs=None):
     """Compile a simple Qt application.
 
@@ -1275,9 +1208,8 @@ def compile_qt_program(name, mname, extra_include_dirs=None, extra_lib_dirs=None
     """
     opengl = (mname == "QtOpenGL")
 
-    qt = [mname]
-    if mname in ("QtAssistant", "QtHelp", "QtOpenGL", "QtWebKit"):
-        qt.append("QtCore")
+    qt = []
+    needed_qt_libs(mname, qt)
 
     makefile = sipconfig.ProgramMakefile(sipcfg, console=1, qt=qt, warnings=0,
             opengl=opengl, debug=opts.debug, arch=sipcfg.arch,
@@ -1754,7 +1686,7 @@ def generate_code(mname, extra_include_dirs=None, extra_lib_dirs=None, extra_lib
 
     if qt_version >= 0x050000 and opts.with_deprecated:
         # The name of this macro is very confusing.
-        makefile.extra_defines.append('QT_DISABLE_DEPRECATED_BEFORE=0x040900')
+        makefile.extra_defines.append('QT_DISABLE_DEPRECATED_BEFORE=0x04ffff')
 
     makefile.generate()
 
@@ -1820,13 +1752,7 @@ def check_license():
 
     # Confirm the license if not already done.
     if not opts.license_confirmed:
-        if ltype == "GPL":
-            loptions = """
-Type '2' to view the GPL v2 license.
-Type '3' to view the GPL v3 license.
-"""
-        else:
-            loptions = """
+        loptions = """
 Type 'L' to view the license.
 """
 
@@ -1855,14 +1781,8 @@ Type 'no' to decline the terms of the license.
             if resp == "no":
                 sys.exit(0)
 
-            if ltype == "GPL":
-                if resp == "2":
-                    os.system("more LICENSE.GPL2")
-                elif resp == "3":
-                    os.system("more LICENSE.GPL3")
-            else:
-                if resp == "l":
-                    os.system("more LICENSE")
+            if resp == "l":
+                os.system("more LICENSE")
 
     # Check that the license file exists and fix its syntax.
     lfile_path = os.path.join(src_dir, "sip", lfile)
@@ -1886,7 +1806,7 @@ def get_build_macros(overrides):
     if "QMAKESPEC" in list(os.environ.keys()):
         fname = os.environ["QMAKESPEC"]
 
-        if not os.path.dirname(fname):
+        if not os.path.dirname(fname) or fname.startswith("unsupported"):
             qt_macx_spec = fname
             fname = os.path.join(qt_archdatadir, "mkspecs", fname)
     elif sys.platform == "darwin":
@@ -2057,12 +1977,10 @@ SOURCES = %s
 #include <QLibraryInfo>
 #include <QTextStream>
 
-// These seem to be missing from the Qt v5 beta.
-#if !defined(QT_EDITION_DESKTOP)
-#define QT_EDITION_DESKTOP      8
-#endif
-#if !defined(QT_EDITION_OPENSOURCE)
-#define QT_EDITION_OPENSOURCE   8
+// This is for Qt v5.  The value isn't accurate but triggers the desired
+// description of the edition.
+#if !defined(QT_EDITION)
+#define QT_EDITION      0x200
 #endif
 
 int main(int argc, char **argv)
@@ -2095,7 +2013,7 @@ int main(int argc, char **argv)
 #if defined(QT_SHARED) || defined(QT_DLL)
     out << "shared\\n";
 #else
-    out << "\\n";
+    out << "static\\n";
 #endif
 
     // Determine which features should be disabled.
@@ -2224,7 +2142,9 @@ int main(int argc, char **argv)
     qt_xfeatures = lines[11:]
 
     if opts.assume_shared:
-        qt_shared = "shared"
+        qt_shared = 'shared'
+    elif qt_shared == 'static':
+        qt_shared = ''
 
     # 'Nokia' is the value that is used by Maemo's version of Qt.
     if qt_licensee in ('Open Source', 'Nokia'):
