@@ -44,7 +44,10 @@
 
 from PyQt4 import QtCore, QtGui
 
-import states_rc
+try:
+    import states_rc3
+except ImportError:
+    import states_rc2
 
 
 class Pixmap(QtGui.QGraphicsObject):
