@@ -513,7 +513,7 @@ class TextEdit(QtGui.QMainWindow):
 
             printer = QtGui.QPrinter(QtGui.QPrinter.HighResolution)
             printer.setOutputFormat(QtGui.QPrinter.PdfFormat)
-            printer.setOutputFileName(fileName)
+            printer.setOutputFileName(fn)
             self.textEdit.document().print_(printer)
 
     def textBold(self):
