@@ -71,12 +71,14 @@ public:
     virtual bool characters( const QString& ch );
     virtual bool fatalError( const QXmlParseException& exception );
     QString language() const { return m_language; }
+    QString sourceLanguage() const { return m_sourceLanguage; }
 
 private:
     MetaTranslator *tor;
     MetaTranslatorMessage::Type type;
     bool inMessage;
     QString m_language;
+    QString m_sourceLanguage;
     QString context;
     QString source;
     QString comment;
@@ -113,6 +115,7 @@ bool TsHandler::startElement( const QString& /* namespaceURI */,
     } else {
         if ( qName == QString("TS") ) {
             m_language = atts.value(QLatin1String("language"));
+            m_sourceLanguage = atts.value(QLatin1String("sourcelanguage"));
         } else if ( qName == QString("context") ) {
             context.truncate( 0 );
             source.truncate( 0 );
@@ -172,9 +175,9 @@ bool TsHandler::endElement( const QString& /* namespaceURI */,
                              QStringList(), true,
                              MetaTranslatorMessage::Unfinished) );
             else
-                tor->insert( MetaTranslatorMessage(context.toAscii(),
+                tor->insert( MetaTranslatorMessage(context.toLatin1(),
                              ContextComment,
-                             accum.toAscii(), QString(), 0,
+                             accum.toLatin1(), QString(), 0,
                              QStringList(), false,
                              MetaTranslatorMessage::Unfinished) );
         }
@@ -190,8 +193,8 @@ bool TsHandler::endElement( const QString& /* namespaceURI */,
                                                comment.toUtf8(), m_fileName, m_lineNumber,
                                                translations, true, type, m_isPlural) );
         else
-            tor->insert( MetaTranslatorMessage(context.toAscii(), source.toAscii(),
-                                               comment.toAscii(), m_fileName, m_lineNumber,
+            tor->insert( MetaTranslatorMessage(context.toLatin1(), source.toLatin1(),
+                                               comment.toLatin1(), m_fileName, m_lineNumber,
                                                translations, false, type, m_isPlural) );
         inMessage = false;
     }
@@ -394,6 +397,7 @@ bool MetaTranslator::load( const QString& filename )
     reader.setErrorHandler( 0 );
 
     m_language = hand->language();
+    m_sourceLanguage = hand->sourceLanguage();
     makeFileNamesAbsolute(QFileInfo(filename).absoluteDir());
 
     delete hand;
@@ -413,9 +417,10 @@ bool MetaTranslator::save( const QString& filename ) const
     //### The xml prolog allows processors to easily detect the correct encoding
     t << "<?xml version=\"1.0\"";
     t << " encoding=\"utf-8\"";
-    t << "?>\n<!DOCTYPE TS><TS version=\"1.1\"";
+    t << "?>\n<!DOCTYPE TS><TS version=\"2.0\"";
     if (!languageCode().isEmpty() && languageCode() != QLatin1String("C"))
-        t << " language=\"" << languageCode() << "\"";
+        t << " language=\"" << languageCode() << "\"" <<\
+            " sourcelanguage=\"" << sourceLanguageCode() << "\"";
     t << ">\n";
     if ( codecName != "ISO-8859-1" )
         t << "<defaultcodec>" << codecName << "</defaultcodec>\n";
@@ -580,11 +585,21 @@ QString MetaTranslator::languageCode() const
     return m_language;
 }
 
+QString MetaTranslator::sourceLanguageCode() const
+{
+    return m_sourceLanguage;
+}
+
 void MetaTranslator::setLanguageCode(const QString &languageCode)
 {
     m_language = languageCode;
 }
 
+void MetaTranslator::setSourceLanguageCode(const QString &languageCode)
+{
+    m_sourceLanguage = languageCode;
+}
+
 bool MetaTranslator::contains( const char *context, const char *sourceText,
                                const char *comment ) const
 {
