@@ -24,7 +24,6 @@
 #include <qfile.h>
 #include <qfileinfo.h>
 #include <qlocale.h>
-#include <qdatetime.h>
 #include <qtextstream.h>
 #include <qbytearray.h>
 #include <qhash.h>
@@ -33,6 +32,14 @@
 #include <qdom.h>
 #include "rcc.h"
 
+
+#if QT_VERSION >= 0x050000
+#define hash_name   qt_hash
+#else
+#define hash_name   qHash
+#endif
+
+
 static bool qt_rcc_write_number(FILE *out, quint32 number, int width)
 {
     int dividend = 1;
@@ -150,7 +157,7 @@ qint64 RCCFileInfo::writeDataName(FILE *out, qint64 offset)
     offset += 2;
 
     //write the hash
-    qt_rcc_write_number(out, qHash(name), 4);
+    qt_rcc_write_number(out, hash_name(name), 4);
     fprintf(out, "\\\n");
     offset += 4;
 
@@ -396,10 +403,10 @@ bool RCCResourceLibrary::output(FILE *out)
 bool
 RCCResourceLibrary::writeHeader(FILE *out)
 {
+    fprintf(out, "# -*- coding: utf-8 -*-\n\n");
     fprintf(out, "# Resource object code\n");
     fprintf(out, "#\n");
-    fprintf(out, "# Created: %s\n", QDateTime::currentDateTime().toString().toLatin1().constData());
-    fprintf(out, "#      by: The Resource Compiler for PyQt (Qt v%s)\n", QT_VERSION_STR);
+    fprintf(out, "# Created by: The Resource Compiler for PyQt4 (Qt v%s)\n", QT_VERSION_STR);
     fprintf(out, "#\n");
     fprintf(out, "# WARNING! All changes made in this file will be lost!\n");
     fprintf(out, "\n");
@@ -411,7 +418,7 @@ RCCResourceLibrary::writeHeader(FILE *out)
 bool
 RCCResourceLibrary::writeDataBlobs(FILE *out)
 {
-    fprintf(out, "qt_resource_data = \"\\\n");
+    fprintf(out, "qt_resource_data = %s\"\\\n", mPrefix);
     QStack<RCCFileInfo*> pending;
 
     if (!root)
@@ -437,7 +444,7 @@ RCCResourceLibrary::writeDataBlobs(FILE *out)
 bool
 RCCResourceLibrary::writeDataNames(FILE *out)
 {
-    fprintf(out, "qt_resource_name = \"\\\n");
+    fprintf(out, "qt_resource_name = %s\"\\\n", mPrefix);
 
     QHash<QString, int> names;
     QStack<RCCFileInfo*> pending;
@@ -468,13 +475,13 @@ RCCResourceLibrary::writeDataNames(FILE *out)
 
 static bool qt_rcc_compare_hash(const RCCFileInfo *left, const RCCFileInfo *right)
 {
-    return qHash(left->name) < qHash(right->name);
+    return hash_name(left->name) < hash_name(right->name);
 }
 
 bool
 RCCResourceLibrary::writeDataStructure(FILE *out)
 {
-    fprintf(out, "qt_resource_struct = \"\\\n");
+    fprintf(out, "qt_resource_struct = %s\"\\\n", mPrefix);
     QStack<RCCFileInfo*> pending;
 
     if (!root)
