@@ -1,3 +1,43 @@
+#############################################################################
+##
+## Copyright (C) 2015 Riverbank Computing Limited.
+## Copyright (C) 2006 Thorsten Marek.
+## All right reserved.
+##
+## This file is part of PyQt.
+##
+## You may use this file under the terms of the GPL v2 or the revised BSD
+## license as follows:
+##
+## "Redistribution and use in source and binary forms, with or without
+## modification, are permitted provided that the following conditions are
+## met:
+##   * Redistributions of source code must retain the above copyright
+##     notice, this list of conditions and the following disclaimer.
+##   * Redistributions in binary form must reproduce the above copyright
+##     notice, this list of conditions and the following disclaimer in
+##     the documentation and/or other materials provided with the
+##     distribution.
+##   * Neither the name of the Riverbank Computing Limited nor the names
+##     of its contributors may be used to endorse or promote products
+##     derived from this software without specific prior written
+##     permission.
+##
+## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+## "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+## LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+## A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+## OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+## SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+## LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+## DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+## THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+## (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+## OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
+##
+#############################################################################
+
+
 import logging
 import os.path
 import sys
@@ -26,6 +66,9 @@ def float_list(prop):
 
 bool_ = lambda v: v == "true"
 
+def qfont_enum(v):
+    return getattr(QtGui.QFont, v)
+
 def needsWidget(func):
     func.needsWidget = True
     return func
@@ -56,20 +99,18 @@ class Properties(object):
     def _pyEnumMember(self, cpp_name):
         try:
             prefix, membername = cpp_name.split("::")
-            DEBUG(membername)
-            if prefix == "Qt":
-                return getattr(QtCore.Qt, membername)
-            else:
-                return getattr(getattr(QtGui, prefix), membername)
         except ValueError:
-            pass
+            prefix = "Qt"
+            membername = cpp_name
 
-        try:
-            return getattr(QtCore.Qt, cpp_name)
-        except AttributeError:
-            # There seems to be a bug where this can succeed when it shouldn't.
-            # If so it will be picked up when the generated code is run.
-            return getattr(getattr(QtGui, self.wclass), cpp_name)
+        if prefix == "Qt":
+            return getattr(QtCore.Qt, membername)
+
+        scope = self.factory.findQObjectType(prefix)
+        if scope is None:
+            raise AttributeError("unknown enum %s" % cpp_name)
+
+        return getattr(scope, membername)
 
     def _set(self, prop):
         expr = [self._pyEnumMember(v) for v in prop.text.split('|')]
@@ -98,14 +139,26 @@ class Properties(object):
         return [self._string(p, notr='true') for p in prop]
 
     def _string(self, prop, notr=None):
-        if prop.get('notr', notr) == 'true':
-            return self._cstring(prop)
+        text = prop.text
 
-        if prop.text is None:
+        if text is None:
             return ""
 
-        return QtGui.QApplication.translate(self.uiname, prop.text, None,
-                QtGui.QApplication.UnicodeUTF8)
+        if prop.get('notr', notr) == 'true':
+            return text
+
+        disambig = prop.get('comment')
+
+        # Allow for Qt5 without deprecated features.
+        try:
+            encoding = QtGui.QApplication.UnicodeUTF8
+            translated = QtGui.QApplication.translate(self.uiname, text,
+                    disambig, encoding)
+        except AttributeError:
+            translated = QtGui.QApplication.translate(self.uiname, text,
+                    disambig)
+
+        return translated
 
     _char = _string
 
@@ -163,9 +216,6 @@ class Properties(object):
         country = getattr(QtCore.QLocale, prop.attrib['country'])
         return QtCore.QLocale(lang, country)
 
-    def _cursor(self, prop):
-        return QtGui.QCursor(QtCore.Qt.CursorShape(int(prop.text)))
-
     def _date(self, prop):
         return QtCore.QDate(*int_list(prop))
 
@@ -292,13 +342,15 @@ class Properties(object):
     _sizepolicy = needsWidget(_sizepolicy)
 
     # font needs special handling/conversion of all child elements.
-    _font_attributes = (("Family",    str),
-                        ("PointSize", int),
-                        ("Weight",    int),
-                        ("Italic",    bool_),
-                        ("Underline", bool_),
-                        ("StrikeOut", bool_),
-                        ("Bold",      bool_))
+    _font_attributes = (("Family",          str),
+                        ("PointSize",       int),
+                        ("Bold",            bool_),
+                        ("Italic",          bool_),
+                        ("Underline",       bool_),
+                        ("Weight",          int),
+                        ("StrikeOut",       bool_),
+                        ("Kerning",         bool_),
+                        ("StyleStrategy",   qfont_enum))
 
     def _font(self, prop):
         newfont = self.factory.createQObject("QFont", "font", (),
@@ -311,8 +363,11 @@ class Properties(object):
             getattr(newfont, "set%s" % (attr,))(converter(v))
         return newfont
 
+    def _cursor(self, prop):
+        return QtGui.QCursor(QtCore.Qt.CursorShape(int(prop.text)))
+
     def _cursorShape(self, prop):
-        return getattr(QtCore.Qt, prop.text)
+        return QtGui.QCursor(getattr(QtCore.Qt, prop.text))
 
     def convert(self, prop, widget=None):
         try:
@@ -342,16 +397,18 @@ class Properties(object):
         return self._getChild("attribute", elem, name, default)
 
     def setProperties(self, widget, elem):
-        try:
-            self.wclass = elem.attrib["class"]
-        except KeyError:
-            pass
-        for prop in elem.findall("property"):
-            prop_name = prop.attrib["name"]
+        # Lines are sunken unless the frame shadow is explicitly set.
+        set_sunken = (elem.attrib.get('class') == 'Line')
+
+        for prop in elem.findall('property'):
+            prop_name = prop.attrib['name']
             DEBUG("setting property %s" % (prop_name,))
 
+            if prop_name == 'frameShadow':
+                set_sunken = False
+
             try:
-                stdset = bool(int(prop.attrib["stdset"]))
+                stdset = bool(int(prop.attrib['stdset']))
             except KeyError:
                 stdset = True
 
@@ -362,7 +419,10 @@ class Properties(object):
             else:
                 prop_value = self.convert(prop, widget)
                 if prop_value is not None:
-                    getattr(widget, "set%s%s" % (ascii_upper(prop_name[0]), prop_name[1:]))(prop_value)
+                    getattr(widget, 'set%s%s' % (ascii_upper(prop_name[0]), prop_name[1:]))(prop_value)
+
+        if set_sunken:
+            widget.setFrameShadow(QtGui.QFrame.Sunken)
 
     # SPECIAL PROPERTIES
     # If a property has a well-known value type but needs special,
@@ -370,41 +430,56 @@ class Properties(object):
 
     # Delayed properties will be set after the whole widget tree has been
     # populated.
-    def _delay(self, widget, prop):
+    def _delayed_property(self, widget, prop):
         prop_value = self.convert(prop)
         if prop_value is not None:
             prop_name = prop.attrib["name"]
-            self.delayed_props.append((
-                getattr(widget, "set%s%s" % (ascii_upper(prop_name[0]), prop_name[1:])),
-                prop_value))
+            self.delayed_props.append((widget, False,
+                    'set%s%s' % (ascii_upper(prop_name[0]), prop_name[1:]),
+                    prop_value))
 
     # These properties will be set with a widget.setProperty call rather than
     # calling the set<property> function.
     def _setViaSetProperty(self, widget, prop):
         prop_value = self.convert(prop)
         if prop_value is not None:
-            widget.setProperty(prop.attrib["name"], prop_value)
+            prop_name = prop.attrib['name']
+
+            # This appears to be a Designer/uic hack where stdset=0 means that
+            # the viewport should be used.
+            if prop[0].tag == 'cursorShape':
+                widget.viewport().setProperty(prop_name, prop_value)
+            else:
+                widget.setProperty(prop_name, prop_value)
 
     # Ignore the property.
     def _ignore(self, widget, prop):
         pass
 
     # Define properties that use the canned handlers.
-    currentIndex = _delay
-    currentRow = _delay
+    currentIndex = _delayed_property
+    currentRow = _delayed_property
 
     showDropIndicator = _setViaSetProperty
     intValue = _setViaSetProperty
     value = _setViaSetProperty
 
     objectName = _ignore
+    margin = _ignore
     leftMargin = _ignore
     topMargin = _ignore
     rightMargin = _ignore
     bottomMargin = _ignore
+    spacing = _ignore
     horizontalSpacing = _ignore
     verticalSpacing = _ignore
 
+    # tabSpacing is actually the spacing property of the widget's layout.
+    def tabSpacing(self, widget, prop):
+        prop_value = self.convert(prop)
+        if prop_value is not None:
+            self.delayed_props.append((widget, True, 'setSpacing', prop_value))
+
     # buddy setting has to be done after the whole widget tree has been
     # populated.  We can't use delay here because we cannot get the actual
     # buddy yet.
@@ -427,11 +502,6 @@ class Properties(object):
             widget.setFrameShape(
                 {"Qt::Horizontal": QtGui.QFrame.HLine,
                  "Qt::Vertical"  : QtGui.QFrame.VLine}[prop[0].text])
-
-            # In Qt Designer, lines appear to be sunken, QFormBuilder loads
-            # them as such, uic generates plain lines.  We stick to the look in
-            # Qt Designer.
-            widget.setFrameShadow(QtGui.QFrame.Sunken)
         else:
             widget.setOrientation(self._enum(prop[0]))
 
@@ -440,13 +510,16 @@ class Properties(object):
     def isWrapping(self, widget, prop):
         widget.setWrapping(self.convert(prop))
 
-    # This is a pseudo-property injected to deal with setContentsMargin()
-    # introduced in Qt v4.3.
-    def pyuicContentsMargins(self, widget, prop):
-        widget.setContentsMargins(*int_list(prop))
+    # This is a pseudo-property injected to deal with margins.
+    def pyuicMargins(self, widget, prop):
+        left, top, right, bottom = int_list(prop)
 
-    # This is a pseudo-property injected to deal with setHorizontalSpacing()
-    # and setVerticalSpacing() introduced in Qt v4.3.
+        if left == top and left == right and left == bottom:
+            widget.setMargin(left)
+        else:
+            widget.setContentsMargins(left, top, right, bottom)
+
+    # This is a pseudo-property injected to deal with spacing.
     def pyuicSpacing(self, widget, prop):
         horiz, vert = int_list(prop)
 
