@@ -1,23 +1,18 @@
 #############################################################################
 ##
-## Copyright (c) 2013 Riverbank Computing Limited <info@riverbankcomputing.com>
+## Copyright (c) 2015 Riverbank Computing Limited <info@riverbankcomputing.com>
 ## 
-## This file is part of PyQt.
+## This file is part of PyQt4.
 ## 
-## This file may be used under the terms of the GNU General Public
-## License versions 2.0 or 3.0 as published by the Free Software
-## Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-## included in the packaging of this file.  Alternatively you may (at
-## your option) use any later version of the GNU General Public
-## License if such license has been publicly approved by Riverbank
-## Computing Limited (or its successors, if any) and the KDE Free Qt
-## Foundation. In addition, as a special exception, Riverbank gives you
-## certain additional rights. These rights are described in the Riverbank
-## GPL Exception version 1.1, which can be found in the file
-## GPL_EXCEPTION.txt in this package.
+## This file may be used under the terms of the GNU General Public License
+## version 3.0 as published by the Free Software Foundation and appearing in
+## the file LICENSE included in the packaging of this file.  Please review the
+## following information to ensure the GNU General Public License version 3.0
+## requirements will be met: http://www.gnu.org/copyleft/gpl.html.
 ## 
-## If you are unsure which license is appropriate for your use, please
-## contact the sales department at sales@riverbankcomputing.com.
+## If you do not wish to use this file under the terms of the GPL version 3.0
+## then you may purchase a commercial license.  For more information contact
+## info@riverbankcomputing.com.
 ## 
 ## This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ## WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -55,7 +50,7 @@ parser.add_option("-d", "--debug", dest="debug", action="store_true",
         default=False, help="show debug output")
 parser.add_option("-i", "--indent", dest="indent", action="store", type="int",
         default=4, metavar="N",
-        help="set indent width to N spaces, tab if N is 0 (default: 4)")
+        help="set indent width to N spaces, tab if N is 0 [default: 4]")
 parser.add_option("-w", "--pyqt3-wrapper", dest="pyqt3_wrapper",
         action="store_true", default=False,
         help="generate a PyQt v3 style wrapper")
@@ -63,6 +58,9 @@ parser.add_option("-w", "--pyqt3-wrapper", dest="pyqt3_wrapper",
 g = optparse.OptionGroup(parser, title="Code generation options")
 g.add_option("--from-imports", dest="from_imports", action="store_true",
         default=False, help="generate imports relative to '.'")
+g.add_option("--resource-suffix", dest="resource_suffix", action="store",
+        type="string", default="_rc", metavar="SUFFIX",
+        help="append SUFFIX to the basename of resource files [default: _rc]")
 parser.add_option_group(g)
 
 opts, args = parser.parse_args()
