@@ -1,24 +1,19 @@
 // This defines the API provided by this library.  It must not be explicitly
 // included by the library itself.
 //
-// Copyright (c) 2013 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2015 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt.
+// This file is part of PyQt4.
 // 
-// This file may be used under the terms of the GNU General Public
-// License versions 2.0 or 3.0 as published by the Free Software
-// Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-// included in the packaging of this file.  Alternatively you may (at
-// your option) use any later version of the GNU General Public
-// License if such license has been publicly approved by Riverbank
-// Computing Limited (or its successors, if any) and the KDE Free Qt
-// Foundation. In addition, as a special exception, Riverbank gives you
-// certain additional rights. These rights are described in the Riverbank
-// GPL Exception version 1.1, which can be found in the file
-// GPL_EXCEPTION.txt in this package.
+// This file may be used under the terms of the GNU General Public License
+// version 3.0 as published by the Free Software Foundation and appearing in
+// the file LICENSE included in the packaging of this file.  Please review the
+// following information to ensure the GNU General Public License version 3.0
+// requirements will be met: http://www.gnu.org/copyleft/gpl.html.
 // 
-// If you are unsure which license is appropriate for your use, please
-// contact the sales department at sales@riverbankcomputing.com.
+// If you do not wish to use this file under the terms of the GPL version 3.0
+// then you may purchase a commercial license.  For more information contact
+// info@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -47,7 +42,12 @@ PyObject *qpycore_pyqtslot(PyObject *args, PyObject *kwds);
 PyObject *qpycore_pyqtsignature(PyObject *args, PyObject *kwds);
 
 // Support for pyqtConfigure().
-int qpycore_pyqtconfigure(PyObject *self, QObject *qobj, PyObject *kwds);
+PyObject *qpycore_pyqtconfigure(PyObject *self, PyObject *args,
+        PyObject *kwds);
+
+// Support for the QObject %FinalisationCode.
+int qpycore_qobject_finalisation(PyObject *self, QObject *qobj, PyObject *kwds,
+        PyObject **updated_kwds);
 
 // Support for converting between PyObject and QString.
 PyObject *qpycore_PyObject_FromQString(const QString &qstr);
@@ -102,6 +102,9 @@ void qpycore_qmetaobject_connectslotsbyname(QObject *qobj,
 // Support for QPyNullVariant.
 QVariant *qpycore_qpynullvariant(PyObject *type);
 
+// Support for pyqt[Set]PickleProtocol().
+extern PyObject *qpycore_pickle_protocol;
+
 // Utilities.
 #if PY_MAJOR_VERSION >= 3
 void qpycore_Unicode_ConcatAndDel(PyObject **string, PyObject *newpart);
