@@ -1,23 +1,18 @@
 // This contains the implementation of various odds and ends.
 //
-// Copyright (c) 2013 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2015 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt.
+// This file is part of PyQt4.
 // 
-// This file may be used under the terms of the GNU General Public
-// License versions 2.0 or 3.0 as published by the Free Software
-// Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-// included in the packaging of this file.  Alternatively you may (at
-// your option) use any later version of the GNU General Public
-// License if such license has been publicly approved by Riverbank
-// Computing Limited (or its successors, if any) and the KDE Free Qt
-// Foundation. In addition, as a special exception, Riverbank gives you
-// certain additional rights. These rights are described in the Riverbank
-// GPL Exception version 1.1, which can be found in the file
-// GPL_EXCEPTION.txt in this package.
+// This file may be used under the terms of the GNU General Public License
+// version 3.0 as published by the Free Software Foundation and appearing in
+// the file LICENSE included in the packaging of this file.  Please review the
+// following information to ensure the GNU General Public License version 3.0
+// requirements will be met: http://www.gnu.org/copyleft/gpl.html.
 // 
-// If you are unsure which license is appropriate for your use, please
-// contact the sales department at sales@riverbankcomputing.com.
+// If you do not wish to use this file under the terms of the GPL version 3.0
+// then you may purchase a commercial license.  For more information contact
+// info@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -72,21 +67,23 @@ char **qpycore_ArgvToC(PyObject *argvlist, int &argc)
     for (int a = 0; a < argc; ++a)
     {
         PyObject *arg_obj = PyList_GET_ITEM(argvlist, a);
-        const char *arg = sipString_AsLatin1String(&arg_obj);
+        char *arg;
 
-        if (arg)
+        if (PyUnicode_Check(arg_obj))
         {
-            arg = qstrdup(arg);
-            Py_DECREF(arg_obj);
+            QByteArray ba_arg = qpycore_PyObject_AsQString(arg_obj).toLocal8Bit();
+            arg = qstrdup(ba_arg.constData());
+        }
+        else if (SIPBytes_Check(arg_obj))
+        {
+            arg = qstrdup(SIPBytes_AS_STRING(arg_obj));
         }
         else
         {
-            // Try not to mess up Qt's argument parsing by simply missing out
-            // the argument.
-            arg = "unknown";
+            arg = const_cast<char *>("invalid");
         }
 
-        argv[a] = argv[a + argc + 1] = const_cast<char *>(arg);
+        argv[a] = argv[a + argc + 1] = arg;
     }
 
     argv[argc + argc + 1] = argv[argc] = NULL;
