@@ -1,23 +1,18 @@
 // This implements the helper for QObject.pyqtConfigure().
 //
-// Copyright (c) 2013 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2015 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt.
+// This file is part of PyQt4.
 // 
-// This file may be used under the terms of the GNU General Public
-// License versions 2.0 or 3.0 as published by the Free Software
-// Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-// included in the packaging of this file.  Alternatively you may (at
-// your option) use any later version of the GNU General Public
-// License if such license has been publicly approved by Riverbank
-// Computing Limited (or its successors, if any) and the KDE Free Qt
-// Foundation. In addition, as a special exception, Riverbank gives you
-// certain additional rights. These rights are described in the Riverbank
-// GPL Exception version 1.1, which can be found in the file
-// GPL_EXCEPTION.txt in this package.
+// This file may be used under the terms of the GNU General Public License
+// version 3.0 as published by the Free Software Foundation and appearing in
+// the file LICENSE included in the packaging of this file.  Please review the
+// following information to ensure the GNU General Public License version 3.0
+// requirements will be met: http://www.gnu.org/copyleft/gpl.html.
 // 
-// If you are unsure which license is appropriate for your use, please
-// contact the sales department at sales@riverbankcomputing.com.
+// If you do not wish to use this file under the terms of the GPL version 3.0
+// then you may purchase a commercial license.  For more information contact
+// info@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -78,8 +73,22 @@ PyObject *qpycore_pyqtconfigure(PyObject *self, PyObject *args, PyObject *kwds)
 
         if (as == AsUnknown)
         {
+#if PY_MAJOR_VERSION >= 3
             PyErr_Format(PyExc_AttributeError,
-                    "'%S' is not a Qt property or a signal", name_obj);
+                    "'%S' is not the name of a Qt property or signal",
+                    name_obj);
+#else
+            PyObject *name_s = PyObject_Str(name_obj);
+   
+            if (name_s != NULL)
+            {
+                PyErr_Format(PyExc_AttributeError,
+                        "'%s' is not the name of a Qt property or signal", 
+                        PyString_AsString(name_s));
+
+                Py_DECREF(name_s);
+            }
+#endif
 
             return 0;
         }
