@@ -1,23 +1,18 @@
 // This contains the implementation of the PyQtProxy class.
 //
-// Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2015 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt.
+// This file is part of PyQt4.
 // 
-// This file may be used under the terms of the GNU General Public
-// License versions 2.0 or 3.0 as published by the Free Software
-// Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-// included in the packaging of this file.  Alternatively you may (at
-// your option) use any later version of the GNU General Public
-// License if such license has been publicly approved by Riverbank
-// Computing Limited (or its successors, if any) and the KDE Free Qt
-// Foundation. In addition, as a special exception, Riverbank gives you
-// certain additional rights. These rights are described in the Riverbank
-// GPL Exception version 1.1, which can be found in the file
-// GPL_EXCEPTION.txt in this package.
+// This file may be used under the terms of the GNU General Public License
+// version 3.0 as published by the Free Software Foundation and appearing in
+// the file LICENSE included in the packaging of this file.  Please review the
+// following information to ensure the GNU General Public License version 3.0
+// requirements will be met: http://www.gnu.org/copyleft/gpl.html.
 // 
-// If you are unsure which license is appropriate for your use, please
-// contact the sales department at sales@riverbankcomputing.com.
+// If you do not wish to use this file under the terms of the GPL version 3.0
+// then you may purchase a commercial license.  For more information contact
+// info@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -31,6 +26,7 @@
 #include <QObject>
 
 #include "qpycore_chimera.h"
+#include "qpycore_qmetaobjectbuilder.h"
 #include "qpycore_pyqtproxy.h"
 #include "qpycore_pyqtpyobject.h"
 #include "qpycore_sip.h"
@@ -42,12 +38,16 @@
 #define PROXY_SLOT_DISABLED 0x40    // The proxy deleteLater() has been called
                                     // or should be called after the slot has
                                     // finished executing.
+#define PROXY_NO_RCVR_CHECK 0x80    // The existence of the receiver C++ object
+                                    // should not be checked.
 
 
 // The last QObject sender.
 QObject *PyQtProxy::last_sender = 0;
 
 
+#if QT_VERSION < 0x050000
+
 static const uint slot_meta_data[] = {
     // content:
     1,       // revision
@@ -58,8 +58,8 @@ static const uint slot_meta_data[] = {
     0,    0, // enums/sets
 
     // slots: signature, parameters, type, tag, flags
-    11,   10,   10,   10, 0x0a,
     21,   10,   10,   10, 0x0a,
+    11,   10,   10,   10, 0x0a,
 
     0        // eod
 };
@@ -79,84 +79,26 @@ const QMetaObject PyQtProxy::staticMetaObject = {
     }
 };
 
+#endif
+
 
 // Create a universal proxy used as a signal.
 PyQtProxy::PyQtProxy(QObject *qtx, const char *sig)
     : QObject(), type(PyQtProxy::ProxySignal), proxy_flags(0),
-            signature(QMetaObject::normalizedSignature(sig))
+            signature(QMetaObject::normalizedSignature(sig)), meta_object(0)
 {
-    // Create a new meta-object on the heap so that it looks like it has a
-    // signal of the right name and signature.
-    sigmo = new QMetaObject;
-    sigmo->d.superdata = &QObject::staticMetaObject;
-    sigmo->d.extradata = 0;
-
-    // Calculate the size of the string meta-data as follows:
-    // - "PyQtProxy" and its terminating '\0' (ie. 9 + 1 bytes),
-    // - a '\0' used for any empty string,
-    // - "disable()" and its terminating '\0' (ie. 9 + 1 bytes),
-    // - the (non-existent) argument names (ie. use the empty string if there
-    //   is less that two arguments, otherwise a comma between each argument
-    //   and the terminating '\0'),
-    // - the name and full signature, less the initial type character, plus the
-    //   terminating '\0'.
-
-    const size_t fixed_len = 9 + 1 + 1 + 9 + 1;
-    const size_t empty_str = 9 + 1;
-
-    int nr_commas = signature.count(',');
-
-    size_t len = fixed_len
-                + (nr_commas >= 0 ? nr_commas + 1 : 0)
-                + signature.size() + 1;
-
-    char *smd = new char[len];
-
-    memcpy(smd, slot_meta_stringdata, fixed_len);
-
-    uint i = fixed_len, args_pos;
-
-    if (nr_commas > 0)
-    {
-        args_pos = i;
-
-        for (int c = 0; c < nr_commas; ++c)
-            smd[i++] = ',';
-
-        smd[i++] = '\0';
-    }
-    else
-        args_pos = empty_str;
-
-    uint sig_pos = i;
-    qstrcpy(&smd[i], signature.constData());
-
-    sigmo->d.stringdata = smd;
-
-    // Add the non-string data.
-    uint *data = new uint[21];
-
-    memcpy(data, slot_meta_data, 21 * sizeof (uint));
-
-    // Replace the second method (ie. unislot()) with the new signal.
-    data[15] = sig_pos;
-    data[16] = args_pos;
-    data[19] = 0x05;
-
-    sigmo->d.data = data;
-
     init(qtx, &proxy_signals, qtx);
 }
 
 
 // Create a universal proxy used as a slot.  Note that this will leak if there
 // is no signal transmitter (ie. no parent) and not marked as single shot.
-// There will be no parsed signature if there was a problem creating the proxy.
+// There will be no meta-object if there was a problem creating the proxy.
 PyQtProxy::PyQtProxy(sipWrapper *txObj, const char *sig, PyObject *rxObj,
         const char *slot, const char **member, int flags)
     : QObject(), type(PyQtProxy::ProxySlot),
             proxy_flags(PROXY_OWNS_SLOT_SIG | flags),
-            signature(QMetaObject::normalizedSignature(sig))
+            signature(QMetaObject::normalizedSignature(sig)), meta_object(0)
 {
     void *tx = 0;
     QObject *qtx = 0;
@@ -200,14 +142,15 @@ PyQtProxy::PyQtProxy(sipWrapper *txObj, const char *sig, PyObject *rxObj,
 
 
 // Create a universal proxy used as a slot being connected to a bound signal.
+// There will be no meta-object if there was a problem creating the proxy.
 PyQtProxy::PyQtProxy(qpycore_pyqtBoundSignal *bs, PyObject *rxObj,
         const char **member)
     : QObject(), type(PyQtProxy::ProxySlot), proxy_flags(0),
-            signature(bs->bound_overload->signature)
+        signature(bs->unbound_signal->signature->signature)
 {
     SIP_BLOCK_THREADS
 
-    real_slot.signature = bs->bound_overload;
+    real_slot.signature = bs->unbound_signal->signature;
 
     // Save the slot.
     if (sipSaveSlot(&real_slot.sip_slot, rxObj, 0) < 0)
@@ -228,6 +171,94 @@ PyQtProxy::PyQtProxy(qpycore_pyqtBoundSignal *bs, PyObject *rxObj,
 // Initialisation common to all ctors.
 void PyQtProxy::init(QObject *qtx, PyQtProxy::ProxyHash *hash, void *key)
 {
+    // Create a new meta-object on the heap so that it looks like it has a
+    // signal of the right name and signature.
+#if QT_VERSION >= 0x050000
+    QMetaObjectBuilder builder;
+
+    builder.setClassName("PyQtProxy");
+    builder.setSuperClass(&QObject::staticMetaObject);
+
+    // Note that signals must be added before slots.
+    if (type == ProxySignal)
+        builder.addSignal(signature);
+    else
+        builder.addSlot("unislot()");
+
+    builder.addSlot("disable()");
+
+    meta_object = builder.toMetaObject();
+#else
+    if (type == ProxySignal)
+    {
+        QMetaObject *mo = new QMetaObject;
+        mo->d.superdata = &QObject::staticMetaObject;
+        mo->d.extradata = 0;
+
+        // Calculate the size of the string meta-data as follows:
+        // - "PyQtProxy" and its terminating '\0' (ie. 9 + 1 bytes),
+        // - a '\0' used for any empty string,
+        // - "disable()" and its terminating '\0' (ie. 9 + 1 bytes),
+        // - the (non-existent) argument names (ie. use the empty string if
+        //   there is less that two arguments, otherwise a comma between each
+        //   argument and the terminating '\0'),
+        // - the name and full signature, less the initial type character, plus
+        //   the terminating '\0'.
+
+        const size_t fixed_len = 9 + 1 + 1 + 9 + 1;
+        const size_t empty_str = 9 + 1;
+
+        int nr_commas = signature.count(',');
+
+        size_t len = fixed_len
+                    + (nr_commas >= 0 ? nr_commas + 1 : 0)
+                    + signature.size() + 1;
+
+        char *smd = new char[len];
+
+        memcpy(smd, slot_meta_stringdata, fixed_len);
+
+        uint i = fixed_len, args_pos;
+
+        if (nr_commas > 0)
+        {
+            args_pos = i;
+
+            for (int c = 0; c < nr_commas; ++c)
+                smd[i++] = ',';
+
+            smd[i++] = '\0';
+        }
+        else
+        {
+            args_pos = empty_str;
+        }
+
+        uint sig_pos = i;
+        qstrcpy(&smd[i], signature.constData());
+
+        mo->d.stringdata = smd;
+
+        // Add the non-string data.
+        uint *data = new uint[21];
+
+        memcpy(data, slot_meta_data, 21 * sizeof (uint));
+
+        // Replace the first method (ie. unislot()) with the new signal.
+        data[10] = sig_pos;
+        data[11] = args_pos;
+        data[14] = 0x05;
+
+        mo->d.data = data;
+
+        meta_object = mo;
+    }
+    else
+    {
+        meta_object = &staticMetaObject;
+    }
+#endif
+
     hashed = true;
     saved_key = key;
     transmitter = qtx;
@@ -296,34 +327,36 @@ PyQtProxy::~PyQtProxy()
         mutex->unlock();
     }
 
-    switch (type)
+    if (type == ProxySlot && real_slot.signature)
     {
-    case ProxySlot:
-        if (real_slot.signature)
+        // Qt can still be tidying up after Python has gone so make sure that
+        // it hasn't.
+        if (Py_IsInitialized())
         {
-            // Qt can still be tidying up after Python has gone so make sure
-            // that it hasn't.
-            if (Py_IsInitialized())
-            {
-                SIP_BLOCK_THREADS
-                sipFreeSipslot(&real_slot.sip_slot);
-                SIP_UNBLOCK_THREADS
-            }
-
-            if (proxy_flags & PROXY_OWNS_SLOT_SIG)
-                delete real_slot.signature;
-
-            real_slot.signature = 0;
+            SIP_BLOCK_THREADS
+            sipFreeSipslot(&real_slot.sip_slot);
+            SIP_UNBLOCK_THREADS
         }
 
-        break;
+        if (proxy_flags & PROXY_OWNS_SLOT_SIG)
+            delete real_slot.signature;
 
-    case ProxySignal:
-        // The casts are needed for MSVC 6.
-        delete[] const_cast<char *>(sigmo->d.stringdata);
-        delete[] const_cast<uint *>(sigmo->d.data);
-        delete sigmo;
-        break;
+        real_slot.signature = 0;
+    }
+
+    if (meta_object)
+    {
+#if QT_VERSION >= 0x050000
+        free(const_cast<QMetaObject *>(meta_object));
+#else
+        if (meta_object != &staticMetaObject)
+        {
+            // The casts are needed for MSVC 6.
+            delete[] const_cast<char *>(meta_object->d.stringdata);
+            delete[] const_cast<uint *>(meta_object->d.data);
+            delete meta_object;
+        }
+#endif
     }
 }
 
@@ -336,10 +369,7 @@ PyQtProxy::ProxyHash PyQtProxy::proxy_signals;
 
 const QMetaObject *PyQtProxy::metaObject() const
 {
-    if (type == ProxySignal)
-        return sigmo;
-
-    return &staticMetaObject;
+    return meta_object;
 }
 
 
@@ -348,7 +378,7 @@ void *PyQtProxy::qt_metacast(const char *_clname)
     if (!_clname)
         return 0;
 
-    if (!qstrcmp(_clname, slot_meta_stringdata))
+    if (qstrcmp(_clname, "PyQtProxy") == 0)
         return static_cast<void *>(const_cast<PyQtProxy *>(this));
 
     return QObject::qt_metacast(_clname);
@@ -367,14 +397,14 @@ int PyQtProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
         switch (_id)
         {
         case 0:
-            disable();
+            if (type == ProxySignal)
+                QMetaObject::activate(this, meta_object, _id, _a);
+            else
+                unislot(_a);
             break;
 
         case 1:
-            if (type == ProxySignal)
-                QMetaObject::activate(this, sigmo, 1, _a);
-            else
-                unislot(_a);
+            disable();
             break;
         }
 
@@ -403,6 +433,7 @@ void PyQtProxy::unislot(void **qargs)
     QObject *saved_last_sender = last_sender;
     last_sender = new_last_sender;
 
+    int no_receiver_check = (proxy_flags & PROXY_NO_RCVR_CHECK);
     PyObject *res;
 
     // See if the sender was a short-circuit signal. */
@@ -411,12 +442,12 @@ void PyQtProxy::unislot(void **qargs)
         // The Python arguments will be the only argument.
         PyObject *pyargs = reinterpret_cast<PyQt_PyObject *>(qargs[1])->pyobject;
 
-        res = sipInvokeSlot(&real_slot.sip_slot, pyargs);
+        res = sipInvokeSlotEx(&real_slot.sip_slot, pyargs, no_receiver_check);
     }
     else
     {
         proxy_flags |= PROXY_SLOT_INVOKED;
-        res = invokeSlot(real_slot, qargs);
+        res = invokeSlot(real_slot, qargs, no_receiver_check);
         proxy_flags &= ~PROXY_SLOT_INVOKED;
 
         // Self destruct if we are a single shot or disabled.
@@ -459,7 +490,8 @@ void PyQtProxy::disable()
 
 
 // Invoke a slot on behalf of C++.
-PyObject *PyQtProxy::invokeSlot(const qpycore_slot &slot, void **qargs)
+PyObject *PyQtProxy::invokeSlot(const qpycore_slot &slot, void **qargs,
+        int no_receiver_check)
 {
     const QList<const Chimera *> &args = slot.signature->parsed_arguments;
 
@@ -486,7 +518,7 @@ PyObject *PyQtProxy::invokeSlot(const qpycore_slot &slot, void **qargs)
     }
 
     // Dispatch to the real slot.
-    PyObject *res = sipInvokeSlot(&slot.sip_slot, argtup);
+    PyObject *res = sipInvokeSlotEx(&slot.sip_slot, argtup, no_receiver_check);
 
     Py_DECREF(argtup);
 
@@ -494,6 +526,14 @@ PyObject *PyQtProxy::invokeSlot(const qpycore_slot &slot, void **qargs)
 }
 
 
+// Disable the check that the receiver C++ object exists before invoking a
+// slot.
+void PyQtProxy::disableReceiverCheck()
+{
+    proxy_flags |= PROXY_NO_RCVR_CHECK;
+}
+
+
 // Find a slot proxy connected to a transmitter.
 PyQtProxy *PyQtProxy::findSlotProxy(void *tx, const char *sig, PyObject *rxObj,
         const char *slot, const char **member)
