@@ -1,23 +1,18 @@
 // This contains the definition of the PyQtProxy class.
 //
-// Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2015 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt.
+// This file is part of PyQt4.
 // 
-// This file may be used under the terms of the GNU General Public
-// License versions 2.0 or 3.0 as published by the Free Software
-// Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-// included in the packaging of this file.  Alternatively you may (at
-// your option) use any later version of the GNU General Public
-// License if such license has been publicly approved by Riverbank
-// Computing Limited (or its successors, if any) and the KDE Free Qt
-// Foundation. In addition, as a special exception, Riverbank gives you
-// certain additional rights. These rights are described in the Riverbank
-// GPL Exception version 1.1, which can be found in the file
-// GPL_EXCEPTION.txt in this package.
+// This file may be used under the terms of the GNU General Public License
+// version 3.0 as published by the Free Software Foundation and appearing in
+// the file LICENSE included in the packaging of this file.  Please review the
+// following information to ensure the GNU General Public License version 3.0
+// requirements will be met: http://www.gnu.org/copyleft/gpl.html.
 // 
-// If you are unsure which license is appropriate for your use, please
-// contact the sales department at sales@riverbankcomputing.com.
+// If you do not wish to use this file under the terms of the GPL version 3.0
+// then you may purchase a commercial license.  For more information contact
+// info@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -78,7 +73,8 @@ public:
     virtual int qt_metacall(QMetaObject::Call, int, void **);
 
     void unislot(void **qargs);
-    static PyObject *invokeSlot(const qpycore_slot &slot, void **qargs);
+    static PyObject *invokeSlot(const qpycore_slot &slot, void **qargs,
+            int no_receiver_check = 0);
     void disable();
 
     int getReceivers(const char *signal) const {return receivers(signal);}
@@ -88,6 +84,8 @@ public:
     static PyQtProxy *findSlotProxy(void *tx, const char *sig, PyObject *rxObj,
             const char *slot, const char **member);
 
+    void disableReceiverCheck();
+
     // The type of a proxy hash.
     typedef QMultiHash<void *, PyQtProxy *> ProxyHash;
 
@@ -126,8 +124,8 @@ private:
     // This object's key in the relevant hash.
     void *saved_key;
 
-    // Only used when type is ProxySignal.
-    QMetaObject *sigmo;
+    // The meta-object.
+    const QMetaObject *meta_object;
 
     PyQtProxy(const PyQtProxy &);
     PyQtProxy &operator=(const PyQtProxy &);
